"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Resolver = void 0;
const node_path_1 = __importDefault(require("node:path"));
const debug_1 = __importDefault(require("debug"));
const typescript_1 = __importDefault(require("typescript"));
const constants_1 = require("./constants");
/* eslint-disable-next-line @typescript-eslint/no-unsafe-assignment, @typescript-eslint/no-unsafe-call*/
const log = (0, debug_1.default)(constants_1.PROJECT_NAME);
function isParentDir(dir, parentToTest) {
    const diff = node_path_1.default.relative(parentToTest, dir);
    return diff.length === 0 || (!diff.startsWith('..') && !node_path_1.default.isAbsolute(diff));
}
class Resolver {
    constructor(ts, compilerOptions, alias) {
        this.ts = ts;
        this.compilerOptions = compilerOptions;
        // Build matchers from compiler options
        this.matchers = [
            (moduleName, sourceFileName) => {
                const retVal = this.resolveModule(sourceFileName, moduleName);
                if (retVal) {
                    log(`${sourceFileName}: Rewrite module ${moduleName} to ${retVal}`);
                    return retVal;
                }
                return null;
            },
        ];
        if (alias) {
            // Build matchers from alias property of the plugin
            this.matchers.push(...Object.entries(alias).map(([from, to]) => {
                const fromRegex = new RegExp(from);
                return function (moduleName, sourceFileName) {
                    if (fromRegex.test(moduleName)) {
                        const retVal = moduleName.replace(fromRegex, to);
                        log(`${sourceFileName}: Rewrite module ${moduleName} to ${retVal} with regex /${from}/ => ${to}`);
                        return retVal;
                    }
                    return null;
                };
            }));
        }
        this.matchers.push();
    }
    resolve(moduleName, sourceFileName) {
        let retVal = null;
        this.matchers.forEach(function (matcher) {
            const result = matcher(moduleName, sourceFileName);
            if (result != null) {
                retVal = result;
            }
        });
        return retVal;
    }
    resolveModule(sourceFilePath, moduleName) {
        const { resolvedModule } = this.ts.resolveModuleName(moduleName, sourceFilePath, this.compilerOptions, this.ts.sys);
        if (!resolvedModule || resolvedModule.packageId) {
            return null;
        }
        const resolvedFile = node_path_1.default.parse(resolvedModule.resolvedFileName);
        const resolvedFileBaseWithoutExt = resolvedFile.name;
        const resolvedFileExt = resolvedFile.ext;
        let resolvedFileDir = resolvedFile.dir;
        let sourceFileDir = node_path_1.default.dirname(sourceFilePath);
        if (this.compilerOptions.rootDirs) {
            let fileRootDir;
            let moduleRootDir;
            this.compilerOptions.rootDirs.forEach(function (rootDir) {
                if (isParentDir(resolvedFileDir, rootDir) && (!moduleRootDir || rootDir.length > moduleRootDir.length)) {
                    moduleRootDir = rootDir;
                }
                if (isParentDir(sourceFileDir, rootDir) && (!fileRootDir || rootDir.length > fileRootDir.length)) {
                    fileRootDir = rootDir;
                }
            });
            if (fileRootDir && moduleRootDir) {
                sourceFileDir = node_path_1.default.relative(fileRootDir, sourceFileDir);
                resolvedFileDir = node_path_1.default.relative(moduleRootDir, resolvedFileDir);
            }
        }
        const normalisedResolvedFileDir = node_path_1.default.relative(sourceFileDir, resolvedFileDir);
        let normalisedResolvedFileBase = resolvedFileBaseWithoutExt;
        if (this.isESM()) {
            if (resolvedFileExt === '.ts') {
                normalisedResolvedFileBase += '.js';
            }
            else if (resolvedFileExt === '.mts') {
                normalisedResolvedFileBase += '.mjs';
            }
            else {
                normalisedResolvedFileBase += resolvedFileExt;
            }
        }
        let normalisedResolvedFilePath = node_path_1.default.join(normalisedResolvedFileDir, normalisedResolvedFileBase);
        if (!normalisedResolvedFilePath.startsWith('.')) {
            normalisedResolvedFilePath = `./${normalisedResolvedFilePath}`;
        }
        return normalisedResolvedFilePath;
    }
    isESM() {
        return (this.compilerOptions.module !== typescript_1.default.ModuleKind.None &&
            this.compilerOptions.module !== typescript_1.default.ModuleKind.CommonJS &&
            this.compilerOptions.module !== typescript_1.default.ModuleKind.AMD &&
            this.compilerOptions.module !== typescript_1.default.ModuleKind.UMD &&
            this.compilerOptions.module !== typescript_1.default.ModuleKind.System);
    }
}
exports.Resolver = Resolver;
//# sourceMappingURL=resolver.js.map